require 'measure_method/logger'
require 'measure_method/metrics'

module MeasureMethod
  def self.included(base)
    base.extend ClassMethods
  end

  module Measurement
    def self.class_name(obj)
      obj.kind_of?(Class) ? obj.name : obj.class.name
    end

    def self.name(obj, method)
      Metrics.metricfy("#{class_name(obj)}.#{method}")
    end
  end

  module ClassMethods
    def measure_methods(*args)
      @methods_to_measure ||= []
      @methods_to_measure += args.map(&:to_sym)
    end

    def method_added(name)
      define_measure_method(name, self)
    end

    def singleton_method_added(name)
      define_measure_method(name, self.singleton_class)
    end

    protected

    def methods_to_measure
      methods = @methods_to_measure || []
      if defined?(superclass.methods_to_measure)
        methods += superclass.methods_to_measure
      end
      methods
    end

    def define_measure_method(name, obj)
      return if @adding_measurers
      @adding_measurers = true
      if methods_to_measure.include? name.to_sym
        unmeasured_name = "unmeasured_#{name}"
        obj.send(:alias_method, unmeasured_name, name)

        obj.send(:define_method, name) do |*args, &block|
          Metrics.measure_block(
            name: Measurement.name(self, name)
          ) { obj.instance_method(unmeasured_name).bind(self).call *args, &block}
        end
      end
      @adding_measurers = nil
    end
  end
end
